@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2013, 2017 MariaDB Corporation AB
+   Copyright (C) 2013, 2018 MariaDB Corporation AB
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,191 +19,105 @@
 #include <ma_odbc.h>
 #include <stdint.h>
 
-#define MADB_FIELD_IS_BINARY(_field) ((_field).charsetnr == BINARY_CHARSETNR)
+#define MADB_FIELD_IS_BINARY(_field) ((_field)->charsetnr == BINARY_CHARSETNR)
 
 void CloseMultiStatements(MADB_Stmt *Stmt)
 {
   unsigned int i;
 
-  for (i=0; i < Stmt->MultiStmtCount; ++i)
+  for (i=0; i < STMT_COUNT(Stmt->Query); ++i)
   {
     MDBUG_C_PRINT(Stmt->Connection, "-->closing %0x", Stmt->MultiStmts[i]);
-    mysql_stmt_close(Stmt->MultiStmts[i]);
+    if (Stmt->MultiStmts[i] != NULL)
+    {
+      mysql_stmt_close(Stmt->MultiStmts[i]);
+    }
   }
   MADB_FREE(Stmt->MultiStmts);
-  Stmt->MultiStmtCount= 0;
   Stmt->stmt= NULL;
 }
 
 
 /* Required, but not sufficient condition */
-BOOL QueryIsPossiblyMultistmt(char *queryStr)
+BOOL QueryIsPossiblyMultistmt(MADB_QUERY *Query)
 {
-  char *semicolon_pos= strchr(queryStr, ';');
-  /* String supposed to come here trimmed. Checking that semicolon is not last char in the string */
-  if (semicolon_pos != NULL && semicolon_pos < queryStr + strlen(queryStr) - 1)
-  {
-    /* CREATE PROCEDURE uses semicolons but is not supported in prepared statement
-        protocol */
-    if (!MADB_IsStatementSupported(queryStr, "CREATE", "PROCEDURE"))
-      return FALSE;
-    if (!MADB_IsStatementSupported(queryStr, "CREATE", "FUNCTION"))
-      return FALSE;
-    if (!MADB_IsStatementSupported(queryStr, "CREATE", "DEFINER"))
-      return FALSE;
-    return TRUE;
-  }
-
-  return FALSE;
+  return Query->QueryType != MADB_QUERY_CREATE_PROC && Query->QueryType != MADB_QUERY_CREATE_FUNC &&
+         Query->QueryType != MADB_QUERY_CREATE_DEFINER;
 }
 
-#define STRING_OR_COMMENT() (quote[0] || quote[1] || comment)
-unsigned int GetMultiStatements(MADB_Stmt *Stmt, char *StmtStr, SQLINTEGER Length)
+
+/* Trims spaces and/or ';' at the end of query */
+int SqlRtrim(char *StmtStr, int Length)
 {
-  char *p, *prev= NULL, wait_char= 0;
-  unsigned int statements= 1;
-  int quote[2]= {0,0}, comment= 0;
-  char *end;
-  MYSQL_STMT *stmt;
-  char *StmtCopy= NULL;
-
-  /* mysql_stmt_init requires locking as well */
-  LOCK_MARIADB(Stmt->Connection);
-  stmt= mysql_stmt_init(Stmt->Connection->mariadb);
-
-  /* if the entire stmt string passes, we don't have multistatement */
-  if (stmt && !mysql_stmt_prepare(stmt, StmtStr, Length))
+  if (Length > 0)
   {
-    mysql_stmt_close(stmt);
-    UNLOCK_MARIADB(Stmt->Connection);
-    return 1;
-  }
-  mysql_stmt_close(stmt);
-
-  /* make sure we don't have trailing whitespace or semicolon */
-  if (Length)
-  {
-    end= StmtStr + Length - 1;
-    while (end > StmtStr && (isspace(*end) || *end == ';'))
+    char *end= StmtStr + Length - 1;
+    while (end > StmtStr && (isspace(0x000000ff & *end) || *end == ';'))
     {
+      *end= '\0';
       --end;
       --Length;
     }
   }
-  /* We can have here not NULL-terminated string as a source, thus we need to allocate, copy meaningful characters and
-     add NULL */
-  p= StmtCopy= MADB_ALLOC(Length + 1);
-  strncpy(StmtCopy, StmtStr, Length);
-  StmtCopy[Length]= '\0';
 
-  end= StmtCopy + Length;
-  
-  while (p < end)
-  {
-    if (wait_char)
-    {
-      if (*p == wait_char && *prev != '\\')
-      {
-        wait_char= 0;
-      }
-    }
-    else
-    {
-      switch (*p) {
-      case '-':
-        if (!STRING_OR_COMMENT() && (p < end - 1) && (char)*(p+1) ==  '-')
-        {
-          wait_char= '\n';
-        }
-        break;
-      case '#':
-        if (!STRING_OR_COMMENT())
-        {
-          wait_char= '\n';
-        }
-        break;
-      case ';':
-        if (!STRING_OR_COMMENT())
-        {
-          statements++;
-          *p= '\0';
-        }
-        break;
-      case '/':
-        if (!STRING_OR_COMMENT() && (p < end - 1) && (char)*(p+1) ==  '*')
-          comment= 1;
-        else if (comment && (p > StmtCopy) && (char)*(prev) == '*')
-          comment= 0;
-        break;
-      case '"':
-        quote[0] = !STRING_OR_COMMENT();
-        break;
-      case '\'':
-        quote[1] = !STRING_OR_COMMENT();
-        break;
-      case '\\':
-        /* *end is \0, so we are safe here to increment by 2 bytes */
-        if ((Stmt->Connection->mariadb->server_status & SERVER_STATUS_NO_BACKSLASH_ESCAPES) == 0  && p < end - 1)
-        {
-          /* Skipping escaped character and resetting prev */
-          p+= 2;
-          prev= 0;
-          continue;
-        }
-      default:
-        break;
-      }
-    }
-
-    prev= p;
-    ++p;
-  }
-
-  if (statements > 1)
-  {
-    int i=0;
-    unsigned int MaxParams= 0;
-
-    p= StmtCopy;
-    Stmt->MultiStmtCount= 0;
-    Stmt->MultiStmtNr= 0;
-    Stmt->MultiStmts= (MYSQL_STMT **)MADB_CALLOC(sizeof(MYSQL_STMT) * statements);
-
-    while (p < StmtCopy + Length)
-    {
-      /* Need to be incremented before CloseMultiStatements() */
-      ++Stmt->MultiStmtCount;
-      Stmt->MultiStmts[i]= i == 0 ? Stmt->stmt : mysql_stmt_init(Stmt->Connection->mariadb);
-      MDBUG_C_PRINT(Stmt->Connection, "-->inited&preparing %0x(%d,%s)", Stmt->MultiStmts[i], i, p);
-
-      if (mysql_stmt_prepare(Stmt->MultiStmts[i], p, (unsigned long)strlen(p)))
-      {
-        MADB_SetNativeError(&Stmt->Error, SQL_HANDLE_STMT, Stmt->MultiStmts[i]);
-        CloseMultiStatements(Stmt);
-        if (StmtCopy)
-        {
-          free(StmtCopy);
-        }
-        UNLOCK_MARIADB(Stmt->Connection);
-
-        return 0;
-      }
-      if (mysql_stmt_param_count(Stmt->MultiStmts[i]) > MaxParams)
-        MaxParams= mysql_stmt_param_count(Stmt->MultiStmts[i]);
-      p+= strlen(p) + 1;
-      ++i;
-    }
-    UNLOCK_MARIADB(Stmt->Connection);
-
-    if (MaxParams)
-      Stmt->params= (MYSQL_BIND *)MADB_CALLOC(sizeof(MYSQL_BIND) * MaxParams);
-  }
-  if (StmtCopy)
-    free(StmtCopy);
-
-  return statements;
+  return Length;
 }
-#undef STRING_OR_COMMENT
+
+
+unsigned int GetMultiStatements(MADB_Stmt *Stmt, BOOL ExecDirect)
+{
+  int          i= 0;
+  unsigned int MaxParams= 0;
+  char        *p= Stmt->Query.RefinedText;
+
+  Stmt->MultiStmtNr= 0;
+  Stmt->MultiStmts= (MYSQL_STMT **)MADB_CALLOC(sizeof(MYSQL_STMT) * STMT_COUNT(Stmt->Query));
+
+  while (p < Stmt->Query.RefinedText + Stmt->Query.RefinedLength)
+  {
+    Stmt->MultiStmts[i]= i == 0 ? Stmt->stmt : mysql_stmt_init(Stmt->Connection->mariadb);
+    MDBUG_C_PRINT(Stmt->Connection, "-->inited&preparing %0x(%d,%s)", Stmt->MultiStmts[i], i, p);
+
+    if (mysql_stmt_prepare(Stmt->MultiStmts[i], p, (unsigned long)strlen(p)))
+    {
+      MADB_SetNativeError(&Stmt->Error, SQL_HANDLE_STMT, Stmt->MultiStmts[i]);
+      CloseMultiStatements(Stmt);
+
+      /* Last paranoid attempt make sure that we did not have a parsing error.
+         More to preserve "backward-compatimility" - we did this before, but before trying to
+         prepare "multi-statement". */
+      if (i == 0 && Stmt->Error.NativeError !=1295 /*ER_UNSUPPORTED_PS*/)
+      {
+        Stmt->stmt= mysql_stmt_init(Stmt->Connection->mariadb);
+        if (mysql_stmt_prepare(Stmt->stmt, STMT_STRING(Stmt), (unsigned long)strlen(STMT_STRING(Stmt))))
+        {
+          mysql_stmt_close(Stmt->stmt);
+          Stmt->stmt= NULL;
+        }
+        else
+        {
+          MADB_DeleteSubqueries(&Stmt->Query);
+          return 0;
+        }
+      }
+      return 1;
+    }
+    if (mysql_stmt_param_count(Stmt->MultiStmts[i]) > MaxParams)
+    {
+      MaxParams= mysql_stmt_param_count(Stmt->MultiStmts[i]);
+    }
+    p+= strlen(p) + 1;
+    ++i;
+  }
+
+  if (MaxParams)
+  {
+    Stmt->params= (MYSQL_BIND *)MADB_CALLOC(sizeof(MYSQL_BIND) * MaxParams);
+  }
+
+  return 0;
+}
+
 
 my_bool MADB_CheckPtrLength(SQLINTEGER MaxLength, char *Ptr, SQLINTEGER NameLen)
 {
@@ -362,16 +276,16 @@ SQLSMALLINT MADB_GetTypeFromConciseType(SQLSMALLINT ConciseType)
 /* }}} */
 
 /* {{{ MADB_GetTypeName */
-char *MADB_GetTypeName(MYSQL_FIELD Field)
+char *MADB_GetTypeName(MYSQL_FIELD *Field)
 {
-  switch(Field.type) {
+  switch(Field->type) {
   case MYSQL_TYPE_DECIMAL:
   case MYSQL_TYPE_NEWDECIMAL:
     return "decimal";
   case MYSQL_TYPE_NULL:
     return "null";
   case MYSQL_TYPE_TINY:
-    return (Field.flags & NUM_FLAG) ? "tinyint" : "char";
+    return (Field->flags & NUM_FLAG) ? "tinyint" : "char";
   case MYSQL_TYPE_SHORT:
     return "smallint";
   case MYSQL_TYPE_LONG:
@@ -480,9 +394,10 @@ char *MADB_GetDefaultColumnValue(MYSQL_RES *res, const char *Column)
   return NULL;
 }
 
-size_t MADB_GetDataSize(MADB_DescRecord *Record, MYSQL_FIELD Field, MARIADB_CHARSET_INFO *charset)
+SQLLEN MADB_GetDataSize(SQLSMALLINT SqlType, SQLLEN OctetLength, BOOL Unsigned,
+                        SQLSMALLINT Precision, SQLSMALLINT Scale, const MARIADB_CHARSET_INFO *Charset)
 {
-  switch(Record->ConciseType)
+  switch(SqlType)
   {
   case SQL_BIT:
     return 1;
@@ -493,7 +408,7 @@ size_t MADB_GetDataSize(MADB_DescRecord *Record, MYSQL_FIELD Field, MARIADB_CHAR
   case SQL_INTEGER:
     return 10;
   case SQL_BIGINT:
-    return 20 - test(Field.flags & UNSIGNED_FLAG);
+    return 20 - test(Unsigned != FALSE);
   case SQL_REAL:
     return 7;
   case SQL_DOUBLE:
@@ -501,45 +416,51 @@ size_t MADB_GetDataSize(MADB_DescRecord *Record, MYSQL_FIELD Field, MARIADB_CHAR
     return 15;
   case SQL_DECIMAL:
   case SQL_NUMERIC:
-    return Record->Precision;
+    return Precision;
   case SQL_TYPE_DATE:
-    return 10;
+    return SQL_DATE_LEN;
   case SQL_TYPE_TIME:
-    return 8 + Field.decimals;
+    return SQL_TIME_LEN + MADB_FRACTIONAL_PART(Scale);
   case SQL_TYPE_TIMESTAMP:
-    return Field.length;
+    return SQL_TIMESTAMP_LEN + MADB_FRACTIONAL_PART(Scale);
+  case SQL_BINARY:
+  case SQL_VARBINARY:
+  case SQL_LONGVARBINARY:
+    return OctetLength;
+  case SQL_GUID:
+    return 36;;
   default:
     {
-      if (MADB_FIELD_IS_BINARY(Field) || charset == NULL || charset->char_maxlen < 2/*i.e.0||1*/)
+      if (Charset == NULL || Charset->char_maxlen < 2/*i.e.0||1*/)
       {
-        return Field.length;
+        return OctetLength;
       }
       else
       {
-        return Field.length/charset->char_maxlen;
+        return OctetLength/Charset->char_maxlen;
       }
     }
   }
 }
 
 /* {{{ MADB_GetDisplaySize */
-size_t MADB_GetDisplaySize(MYSQL_FIELD Field, MARIADB_CHARSET_INFO *charset)
+size_t MADB_GetDisplaySize(MYSQL_FIELD *Field, MARIADB_CHARSET_INFO *charset)
 {
   /* Todo: check these values with output from mysql --with-columntype-info */
-  switch (Field.type) {
+  switch (Field->type) {
   case MYSQL_TYPE_NULL:
     return 1;
   case MYSQL_TYPE_BIT:
-    return (Field.length == 1) ? 1 : (Field.length + 7) / 8 * 2;
+    return (Field->length == 1) ? 1 : (Field->length + 7) / 8 * 2;
   case MYSQL_TYPE_TINY:
-    return 4 - test(Field.flags & UNSIGNED_FLAG);
+    return 4 - test(Field->flags & UNSIGNED_FLAG);
   case MYSQL_TYPE_SHORT:
   case MYSQL_TYPE_YEAR:
-    return 6 - test(Field.flags & UNSIGNED_FLAG);
+    return 6 - test(Field->flags & UNSIGNED_FLAG);
   case MYSQL_TYPE_INT24:
-    return 9 - test(Field.flags & UNSIGNED_FLAG);
+    return 9 - test(Field->flags & UNSIGNED_FLAG);
   case MYSQL_TYPE_LONG:
-    return 11 - test(Field.flags & UNSIGNED_FLAG);
+    return 11 - test(Field->flags & UNSIGNED_FLAG);
   case MYSQL_TYPE_LONGLONG:
     return 20;
   case MYSQL_TYPE_DOUBLE:
@@ -548,15 +469,15 @@ size_t MADB_GetDisplaySize(MYSQL_FIELD Field, MARIADB_CHARSET_INFO *charset)
     return 7;
   case MYSQL_TYPE_DECIMAL:
   case MYSQL_TYPE_NEWDECIMAL:
-    return 10;
+    return Field->length;
   case MYSQL_TYPE_DATE:
-    return 10; /* YYYY-MM-DD */
+    return SQL_DATE_LEN; /* YYYY-MM-DD */
   case MYSQL_TYPE_TIME:
-    return 8; /* HH:MM:SS */
+    return SQL_TIME_LEN + MADB_FRACTIONAL_PART(Field->decimals); /* HH:MM:SS.ffffff */
   case MYSQL_TYPE_NEWDATE:
   case MYSQL_TYPE_TIMESTAMP:
   case MYSQL_TYPE_DATETIME:
-    return 19;
+    return SQL_TIMESTAMP_LEN + MADB_FRACTIONAL_PART(Field->decimals);
   case MYSQL_TYPE_BLOB:
   case MYSQL_TYPE_ENUM:
   case MYSQL_TYPE_GEOMETRY:
@@ -570,15 +491,15 @@ size_t MADB_GetDisplaySize(MYSQL_FIELD Field, MARIADB_CHARSET_INFO *charset)
   {
     if (MADB_FIELD_IS_BINARY(Field))
     {
-      return Field.length*2; /* ODBC specs says we should give 2 characters per byte to display binaray data in hex form */
+      return Field->length*2; /* ODBC specs says we should give 2 characters per byte to display binaray data in hex form */
     }
     else if (charset == NULL || charset->char_maxlen < 2/*i.e.0||1*/)
     {
-      return Field.length;
+      return Field->length;
     }
     else
     {
-      return Field.length/charset->char_maxlen;
+      return Field->length/charset->char_maxlen;
     }
   }
   default:
@@ -588,15 +509,15 @@ size_t MADB_GetDisplaySize(MYSQL_FIELD Field, MARIADB_CHARSET_INFO *charset)
 /* }}} */
 
 /* {{{ MADB_GetOctetLength */
-size_t MADB_GetOctetLength(MYSQL_FIELD Field, unsigned short MaxCharLen)
+size_t MADB_GetOctetLength(MYSQL_FIELD *Field, unsigned short MaxCharLen)
 {
-  size_t Length= MIN(MADB_INT_MAX32, Field.length);
+  size_t Length= MIN(MADB_INT_MAX32, Field->length);
 
-  switch (Field.type) {
+  switch (Field->type) {
   case MYSQL_TYPE_NULL:
     return 1;
   case MYSQL_TYPE_BIT:
-    return (Field.length + 7) / 8;
+    return (Field->length + 7) / 8;
   case MYSQL_TYPE_TINY:
     return 1;
   case MYSQL_TYPE_YEAR:
@@ -614,15 +535,15 @@ size_t MADB_GetOctetLength(MYSQL_FIELD Field, unsigned short MaxCharLen)
     return 4;
   case MYSQL_TYPE_DECIMAL:
   case MYSQL_TYPE_NEWDECIMAL:
-    return Field.length;
+    return Field->length;
   case MYSQL_TYPE_DATE:
-    return SQL_DATE_LEN;
+    return sizeof(SQL_DATE_STRUCT);
   case MYSQL_TYPE_TIME:
-    return SQL_TIME_LEN;
+    return sizeof(SQL_TIME_STRUCT);
    case MYSQL_TYPE_NEWDATE:
   case MYSQL_TYPE_TIMESTAMP:
   case MYSQL_TYPE_DATETIME:
-    return SQL_TIMESTAMP_LEN;
+    return sizeof(SQL_TIMESTAMP_STRUCT);
   case MYSQL_TYPE_BLOB:
   case MYSQL_TYPE_ENUM:
   case MYSQL_TYPE_GEOMETRY:
@@ -634,7 +555,7 @@ size_t MADB_GetOctetLength(MYSQL_FIELD Field, unsigned short MaxCharLen)
   case MYSQL_TYPE_STRING:
   case MYSQL_TYPE_VARCHAR:
   case MYSQL_TYPE_VAR_STRING:
-    /*if (!(Field.flags & BINARY_FLAG))
+    /*if (!(Field->flags & BINARY_FLAG))
       Length *= MaxCharLen ? MaxCharLen : 1;*/
     return Length;
   default:
@@ -695,9 +616,9 @@ int MADB_GetDefaultType(int SQLDataType)
 }
 /* }}} */
 
-/* {{{ MADB_GetODBCType */
+/* {{{ MapMariadDbToOdbcType */
        /* It's not quite right to mix here C and SQL types, even though constants are sort of equal */
-SQLSMALLINT MADB_GetODBCType(MYSQL_FIELD *field)
+SQLSMALLINT MapMariadDbToOdbcType(MYSQL_FIELD *field)
 {
   switch (field->type) {
     case MYSQL_TYPE_BIT:
@@ -730,13 +651,13 @@ SQLSMALLINT MADB_GetODBCType(MYSQL_FIELD *field)
     case MYSQL_TYPE_BLOB:
     case MYSQL_TYPE_MEDIUM_BLOB:
     case MYSQL_TYPE_LONG_BLOB:
-      return MADB_FIELD_IS_BINARY(*field) ? SQL_LONGVARBINARY : SQL_LONGVARCHAR;
+      return MADB_FIELD_IS_BINARY(field) ? SQL_LONGVARBINARY : SQL_LONGVARCHAR;
     case MYSQL_TYPE_LONGLONG:
       return SQL_BIGINT;
     case MYSQL_TYPE_STRING:
-      return MADB_FIELD_IS_BINARY(*field) ? SQL_BINARY : SQL_CHAR;
+      return MADB_FIELD_IS_BINARY(field) ? SQL_BINARY : SQL_CHAR;
     case MYSQL_TYPE_VAR_STRING:
-      return MADB_FIELD_IS_BINARY(*field) ? SQL_VARBINARY : SQL_VARCHAR;
+      return MADB_FIELD_IS_BINARY(field) ? SQL_VARBINARY : SQL_VARCHAR;
     case MYSQL_TYPE_SET:
     case MYSQL_TYPE_ENUM:
       return SQL_CHAR;
@@ -863,35 +784,29 @@ int MADB_GetMaDBTypeAndLength(SQLINTEGER SqlDataType, my_bool *Unsigned, unsigne
 SQLRETURN MADB_CopyMadbTimestamp(MADB_Stmt *Stmt, MYSQL_TIME *tm, MADB_Desc *Ard, MADB_DescRecord *ArdRecord, int Type, unsigned long RowNumber)
 {
   void *DataPtr= GetBindOffset(Ard, ArdRecord, ArdRecord->DataPtr, RowNumber, ArdRecord->OctetLength);
+  SQLLEN Dummy, *Length= ArdRecord->OctetLengthPtr ? ArdRecord->OctetLengthPtr : &Dummy,
+                *Ind= ArdRecord->IndicatorPtr ? ArdRecord->IndicatorPtr :&Dummy;
 
-  switch(Type) {
-  case SQL_C_TIMESTAMP:
-  case SQL_C_TYPE_TIMESTAMP:
+  switch(Type)
+  {
+    case SQL_C_TIMESTAMP:
+    case SQL_C_TYPE_TIMESTAMP:
     {
       SQL_TIMESTAMP_STRUCT *ts= (SQL_TIMESTAMP_STRUCT *)DataPtr;
-      /* if (!tm->year)
-      {
-        time_t sec_time;
-        struct tm * cur_tm;
-        sec_time= time(NULL);
-        cur_tm= localtime(&sec_time);
-        ts->year= 1900 + cur_tm->tm_year;
-        ts->month= cur_tm->tm_mon + 1;
-        ts->day= cur_tm->tm_mday;
-      }
-      else */
-      {
-        ts->year= tm->year;
-        ts->month= tm->month;
-        ts->day= tm->day;
-      }
+
+      ts->year= tm->year;
+      ts->month= tm->month;
+      ts->day= tm->day;
       ts->hour= tm->hour;
       ts->minute= tm->minute;
       ts->second= tm->second;
       ts->fraction= tm->second_part * 1000;
+      *Length= sizeof(SQL_TIMESTAMP_STRUCT);
+
       if (ts->year + ts->month + ts->day + ts->hour + ts->minute + ts->fraction + ts->second == 0)
-        if (ArdRecord->IndicatorPtr)
-          *ArdRecord->IndicatorPtr= SQL_NULL_DATA;
+      {
+        *Ind= SQL_NULL_DATA;
+	    }
     }
     break;
     case SQL_C_TIME:
@@ -906,6 +821,7 @@ SQLRETURN MADB_CopyMadbTimestamp(MADB_Stmt *Stmt, MYSQL_TIME *tm, MADB_Desc *Ard
       ts->hour= tm->hour;
       ts->minute= tm->minute;
       ts->second= tm->second;
+      *Length= sizeof(SQL_TIME_STRUCT);
     }
     break;
     case SQL_C_DATE:
@@ -915,13 +831,16 @@ SQLRETURN MADB_CopyMadbTimestamp(MADB_Stmt *Stmt, MYSQL_TIME *tm, MADB_Desc *Ard
       ts->year= tm->year;
       ts->month= tm->month;
       ts->day= tm->day;
+      *Length= sizeof(SQL_DATE_STRUCT);
+
       if (ts->year + ts->month + ts->day == 0)
-        if (ArdRecord->IndicatorPtr)
-          *ArdRecord->IndicatorPtr= SQL_NULL_DATA;
+      {
+        *Ind= SQL_NULL_DATA;
+      }
     }
     break;
   }
-
+    
   return SQL_SUCCESS;
 }
 
@@ -973,10 +892,11 @@ void MADB_NumericInit(SQL_NUMERIC_STRUCT *number, MADB_DescRecord *Ard)
 }
 
 /* {{{ MADB_CharToSQLNumeric */
-int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdRecord, unsigned long RowNumber)
+int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdRecord, SQL_NUMERIC_STRUCT *dst_buffer, unsigned long RowNumber)
 {
   char *p;
-  SQL_NUMERIC_STRUCT *number= (SQL_NUMERIC_STRUCT *)GetBindOffset(Ard, ArdRecord, ArdRecord->DataPtr, RowNumber, ArdRecord->OctetLength);
+  SQL_NUMERIC_STRUCT *number= dst_buffer != NULL ? dst_buffer :
+    (SQL_NUMERIC_STRUCT *)GetBindOffset(Ard, ArdRecord, ArdRecord->DataPtr, RowNumber, ArdRecord->OctetLength);
   int ret= 0;
 
   if (!buffer || !number)
@@ -994,9 +914,11 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
   {
     number->precision= MADB_DEFAULT_PRECISION;
   }
-   
+
   while (*p=='0')
+  {
     p++;
+  }
   if (*p)
   {
     int i;
@@ -1021,17 +943,18 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       while (*p)
       {
         /* ignore non numbers */
-        if (!isdigit(*p))
+        if (!isdigit(0x000000ff & *p))
           break;
         digits_total++;
         /* ignore trailing zeros */
         if (*p != '0')
+        {
           digits_significant= digits_total;
+        }
         p++;
       }
-      /* check possible overflow */
-      digits_significant= MIN(digits_significant,number->scale);
-      if (digits_count + digits_significant > number->precision)
+
+      if (digits_count + number->scale > number->precision)
       {
         int i;
         /* if digits are zero there is no overflow */
@@ -1044,12 +967,20 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       }
       
       memcpy(digits + digits_count, dot + 1, digits_significant);
+      if (number->scale > digits_significant)
+      {
+        for (i= digits_count + digits_significant; i < number->precision && i < digits_count +number->scale; ++i)
+        {
+          digits[i]= '0';
+        }
+        digits_significant= number->scale;
+      }
       digits_count+= digits_significant;
-      number->scale= (SQLSCHAR)digits_significant;
-    } else 
+    }
+    else 
     {
       char *start= p;
-      while (*p && isdigit(*p))
+      while (*p && isdigit(0x000000ff & *p))
         p++;
       /* check overflow */
       if (p - start > number->precision)
@@ -1060,8 +991,8 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       memcpy(digits, start, digits_count);
       number->scale= ArdRecord->Scale ? ArdRecord->Scale : 0;
     }
+
     /* Rounding */
-    
     if (number->scale < 0)
     {
       int64_t OldVal, Val;
@@ -1079,7 +1010,8 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       if (digits_count > number->precision)
         return MADB_ERR_22003;
     }
-    digits_count= MIN(digits_count, 38);
+
+    digits_count= MIN(digits_count, MADB_DEFAULT_PRECISION);
     for (hval = 0, bit = 1L, sta = 0, olen = 0; sta < digits_count;)
     {
       for (dig = 0, i = sta; i < digits_count; i++)
@@ -1107,7 +1039,9 @@ int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdReco
       } 
     }
     if (hval && olen < SQL_MAX_NUMERIC_LEN - 1)
+    {
       number->val[olen++] = hval;
+    }
   } 
   return ret;
 }
