@@ -83,6 +83,30 @@ int MADB_ResetParser(MADB_Stmt *Stmt, char *OriginalQuery, SQLINTEGER OriginalLe
   return 0;
 }
 
+void MADB_DeleteSubqueries(MADB_QUERY * Query)
+{
+  unsigned int i;
+  SINGLE_QUERY SubQuery;
+
+  for (i= 0; i < Query->SubQuery.elements; ++i)
+  {
+    MADB_GetDynamic(&Query->SubQuery, (char *)&SubQuery, i);
+    MADB_DeleteDynamic(&SubQuery.ParamPos);
+  }
+  MADB_DeleteDynamic(&Query->SubQuery);
+}
+
+void MADB_AddSubQuery(MADB_QUERY * Query, char * SubQueryText, enum enum_madb_query_type QueryType)
+{
+  SINGLE_QUERY SubQuery;
+
+  SubQuery.QueryText= SubQueryText;
+  SubQuery.QueryType= QueryType;
+  MADB_InitDynamicArray(&SubQuery.ParamPos, sizeof(unsigned int), 20, 20);
+
+  MADB_InsertDynamic(&Query->SubQuery, (char*)&SubQuery);
+}
+
 
 void MADB_DeleteQuery(MADB_QUERY *Query)
 {
@@ -90,6 +114,8 @@ void MADB_DeleteQuery(MADB_QUERY *Query)
   MADB_FREE(Query->Original);
   MADB_DeleteDynamic(&Query->Tokens);
 
+  MADB_DeleteSubqueries(Query);
+
   memset(Query, 0, sizeof(MADB_QUERY));
 }
 
@@ -108,6 +134,7 @@ int MADB_ParseQuery(MADB_QUERY * Query)
   return ParseQuery(Query);
 }
 
+/*----------------- Tokens stuff ------------------*/
 
 char *MADB_Token(MADB_QUERY *Query, unsigned int Idx)
 {
@@ -174,19 +201,12 @@ char *MADB_ParseCursorName(MADB_QUERY *Query, unsigned int *Offset)
 }
 
 
-/* Not used - rather a placeholder in case we need it */
-const char * MADB_FindParamPlaceholder(MADB_Stmt *Stmt)
-{ 
-  return STMT_STRING(Stmt);
-}
-
-
 enum enum_madb_query_type MADB_GetQueryType(const char *Token1, const char *Token2)
 {
   /* We need for the case when MS Access adds parenthesis around query - see ODBC-57*/
   while (*Token1 && !isalpha(*Token1))
     ++Token1;
-  if (_strnicmp(Token1, "SELECT", 6) == 0)
+  if (_strnicmp(Token1, "SELECT", 6) == 0 || _strnicmp(Token1, "WITH", 4) == 0)
   {
     return MADB_QUERY_SELECT;
   }
@@ -252,9 +272,20 @@ enum enum_madb_query_type MADB_GetQueryType(const char *Token1, const char *Toke
       return MADB_QUERY_SET;
     }
   }
+  if (_strnicmp(Token1, "DESC", 4) == 0)
+  {
+    return MADB_QUERY_DESCRIBE;
+  }
   return MADB_QUERY_NO_RESULT;
 }
 
+/* -------------------- Tokens - End ----------------- */
+
+/* Not used - rather a placeholder in case we need it */
+const char * MADB_FindParamPlaceholder(MADB_Stmt *Stmt)
+{
+  return STMT_STRING(Stmt);
+}
 
 /* Function assumes that query string has been trimmed */
 char* FixIsoFormat(char * StmtString, size_t *Length)
@@ -284,10 +315,11 @@ int ParseQuery(MADB_QUERY *Query)
   BOOL         ReadingToken= FALSE;
   unsigned int Offset, StmtTokensCount= 0;
   size_t       Length= Query->RefinedLength;
-  char        *end= p + Length;
+  char        *end= p + Length, *CurQuery= NULL, *LastSemicolon= NULL;
   enum enum_madb_query_type StmtType;
 
-  MADB_InitDynamicArray(&Query->Tokens, sizeof(unsigned int), 20, 20);
+  MADB_InitDynamicArray(&Query->Tokens, sizeof(unsigned int), 20, 40);
+  MADB_InitDynamicArray(&Query->SubQuery, sizeof(SINGLE_QUERY), 20, 40);
 
   while (p < end)
   {
@@ -304,7 +336,7 @@ int ParseQuery(MADB_QUERY *Query)
       /* On saving 1st statement's token, we are incrementing statements counter */
       if (StmtTokensCount == 1)
       {
-        ++Query->MultiStmtCount;
+        CurQuery= p;
       }
       /* Having 2 first tockens we can get statement type. And we need to know it for the case of multistatement -
          some statements may "legally" contain ';' */
@@ -313,6 +345,8 @@ int ParseQuery(MADB_QUERY *Query)
         /* We are currently at 2nd token of statement, and getting previous token position from Tokens array*/
         StmtType= MADB_GetQueryType(MADB_Token(Query, Query->Tokens.elements - 2), p);
         Query->ReturnsResult= Query->ReturnsResult || !QUERY_DOESNT_RETURN_RESULT(StmtType);
+        MADB_AddSubQuery(Query, CurQuery, StmtType);
+
         /* If we on first statement, setting QueryType*/
         if (Query->Tokens.elements == 2)
         {
